#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Burger string functions

Package that contains file manipulation functions
"""

## \package burger.fileutils


from __future__ import absolute_import, print_function, unicode_literals

import errno
import os
import shutil
import stat
from .strutils import is_string
from .buildutils import perforce_edit, host_machine, get_windows_host_type

########################################


def create_folder_if_needed(path):
	"""
	Given a pathname to a folder, detect if the folder exists, if not, create it.

	Call os.makedirs(path) but does not throw an
	exception if the directory already exists.

	Args:
		path: A string object with the pathname.
	"""

	try:
		os.makedirs(path)
	except OSError as error:
		if error.errno != errno.EEXIST:
			raise

########################################


def delete_file(filename):
	"""
	Given a pathname to a file, delete it.

	If the file doesn't exist, it will return without raising
	an exception.

	Args:
		filename: A string object with the filename
	"""

	try:
		os.remove(filename)
	except OSError as error:
		if error.errno != errno.ENOENT:
			raise

########################################


def is_source_newer(source, destination):
	"""
	Return False if the source file is older then the destination file

	Check the modification times of both files to determine if the
	source file is newer

	Return False if destination is newer, not False if not.

	Args:
		source: string pathname of the file to test
		destination: string pathname of the file to test against

	Returns:
		False if not newer, True if newer, 2 if there is no source file
	"""

	# Get the source file's modification time, If there's no source file, return 2
	try:
		srctime = os.path.getmtime(source)
	except OSError as error:
		if error.errno != errno.ENOENT:
			raise
		return 2

	# Get the destination file's modification time, if missing return True
	try:
		desttime = os.path.getmtime(destination)
	except OSError as error:
		if error.errno != errno.ENOENT:
			raise
		return True

	# Is the source older or equal? Return False to not update the destination
	if srctime <= desttime:
		return False
	return True

########################################


def copy_file_if_needed(source, destination, silent=False):
	"""
	Copy a file only if newer

	Copy a file only if the destination is missing or is older than the source
	file

	Args:
		source: string pathname of the file to copy from
		destination: string pathname of the file to copy to
		silent: True if print output is suppressed

	Returns:
		Zero if no error otherwise non-zero
	"""

	# If there is a destination file, check the modification times

	if is_source_newer(source, destination) is True:

		# Copy the file

		if silent is not False:
			print('Copying {0} -> {1}'.format(source, destination))
		try:
			shutil.copyfile(source, destination)
		except IOError as error:
			print(error)
			return 2
	return 0

########################################


def copy_file_checkout_if_needed(source, destination, silent=False):

	"""
	Copy a file only if newer and alert Perforce if present

	Copy a file only if needed and check out the destination file before the copy
	operation if Perforce is detected

	Args:
		source: string pathname of the file to copy from
		destination: string pathname of the file to copy to
		silent: True if print output is suppressed

	Returns:
		Zero on no error, non-zero on error

	See:
		perforce_edit()
	"""

	if is_source_newer(source, destination) is True:

		# Alert perforce that the file is to be modified

		perforce_edit(destination)

		# Copy the file
		if silent is not False:
			print('Copying {} -> {}'.format(source, destination))
		try:
			shutil.copyfile(source, destination)
		except IOError as error:
			print(error)
			return 2
	return 0

########################################


def copy_directory_if_needed(source, destination, exception_list=None, \
	silent=False):

	"""
	Copy all of the files in a directory into a new directory

	Creating any necessary directories in the process, and it
	will skip files with specific extensions

	Note:
		This is a recursive function

	Args:
		source: string pathname of the directory to copy from
		destination: string pathname of the directory to copy to
		exception_list: optional list of file extensions to ignore during copy
		silent: True if print output is suppressed

	Returns:
		Zero if no error, non-zero on error

	See:
		copy_file_if_needed(), create_folder_if_needed()
	"""

	# Ensure there is an exception list, even if empty
	if exception_list is None:
		exception_list = []

	# Make sure the output folder exists
	create_folder_if_needed(destination)

	name_list = os.listdir(source)
	for base_name in name_list:
		for item in exception_list:
			if base_name.endswith(item):
				break
		else:
			file_name = os.path.join(source, base_name)

			# Handle the directories found
			if os.path.isdir(file_name):
				# Recursive!
				error = copy_directory_if_needed(file_name, \
					os.path.join(destination, base_name), exception_list, silent)
			else:
				error = copy_file_if_needed(file_name, os.path.join( \
					destination, base_name), silent)

			# Exit immediately on error
			if error != 0:
				return error

	return 0

########################################


def shutil_readonly_cb(func, path, exception_info):

	"""
	Subroutine for shutil.rmtree() to delete read only files

	shutil.rmtree() raises an exception if there are read
	only files in the directory being deleted. Use this
	callback to allow read only files to be disposed of.

	Examples:
		import burger
		import shutil

		shutil.rmtree(PATH_TO_DIRECTORY,onerror = burger.shutil_readonly_cb)

	Note:
		This is a callback function

	Args:
		func: Not used
		path: pathname of the file that is read only
		exception_info: Information about the exception

	See:
		delete_directory()
	"""

	# File not found? Ignore
	value = exception_info[1]
	if value.args[0] == errno.ENOENT:
		return

	# Read only?
	# EACCESS for Linux/MacOSX, EIO for Windows

	if value.args[0] == errno.EACCES or value.args[0] == errno.EIO:

		# Mark as writable and try again to delete the file
		os.chmod(path, stat.S_IWRITE)
		os.unlink(path)

########################################


def delete_directory(path, delete_read_only=False):
	"""
	Recursively delete a directory

	Delete a directory and all of the files and directories within.

	Args:
		path: Pathname of the directory to delete
		delete_read_only: True if read only files are to be deleted as well

	See:
		shutil_readonly_cb()
	"""

	if delete_read_only is False:
		shutil.rmtree(path, ignore_errors=True)
	else:
		shutil.rmtree(path, onerror=shutil_readonly_cb)

########################################


def get_tool_path(tool_folder, tool_name, encapsulate=False):

	"""
	Find executable tool directory

	For allowing builds on multiple operating system hosts under the Burgerlib
	way of project management, it's necessary to query what is the host
	operating system and glean out which folder to find a executable compiled
	for that specific host

	Args:
		tool_folder: Pathname to the folder that contains the executables
		tool_name: Bare name of the tool (Windows will append '.exe')
		encapsulate: False if a path is requested, True if it's quoted to be
			used as a string to be sent to command line shell

	Returns:
		Full pathname to the tool to execute
	"""

	host = host_machine()

	# Macosx uses fat binaries
	if host == 'macosx':
		exename = os.path.join(tool_folder, 'macosx', tool_name)

	# Linux is currently just 64 bit Intel, will have to update
	# as more platforms are supported
	elif host == 'linux':
		exename = os.path.join(tool_folder, 'linux', tool_name)

	# Windows supports 32 and 64 bit Intel
	elif host == 'windows':
		exename = os.path.join(tool_folder, 'windows', get_windows_host_type(), \
			tool_name + '.exe')
	else:

		# On unknown platforms, assume the tool is in the path for the fallback
		exename = tool_name

	# Encase in quotes to handle spaces in filenames

	if encapsulate is True:
		return '"{}"'.format(exename)
	return exename

########################################


def traverse_directory( \
	working_dir, filename_list, terminate=False, find_directory=False):

	"""
	Create a list of all copies of a file following a directory

	Starting with a working directory, test if a file exists
	and if so, insert it into a list. The list will be
	starting from the root with the last entry
	being at the working directory

	Args:
		working_dir: string with the path of the folder to start the search
		filename_list: string or an iterable of strings with the name(s)
			of the file(s) to find in the scanned folders
		terminate: True if searching will end on the first found file
		find_directory: True if searching for directories instead of files.
	Returns:
		List of pathnames (With filename appended)
	"""

	# Ensure that if the input was a string, that it becomes
	# an iterable to work below
	if is_string(filename_list):
		filename_list = (filename_list,)

	# Convert into a unpacked pathname
	tempdir = os.path.abspath(working_dir)

	dirlist = []

	# Loop
	while 1:
		# Iterate over the list and detect if these files are present
		for item in filename_list:
			temppath = os.path.join(tempdir, item)
			if find_directory:
				if os.path.isdir(temppath):
					# Insert at the beginning
					dirlist.insert(0, temppath)
					if terminate:
						return dirlist

			elif os.path.isfile(temppath):
				# Insert at the beginning
				dirlist.insert(0, temppath)
				if terminate:
					return dirlist

		# Pop a folder
		tempdir2 = os.path.dirname(tempdir)
		# Already at the top of the directory?
		if tempdir2 is None or tempdir2 == tempdir:
			break
		# Use the new folder
		tempdir = tempdir2

	# Return the list of files
	return dirlist

########################################


def unlock_files(working_dir, recursive=False):
	"""
	Iterate over a directory and unlock all read-only files.

	This function will generate a list of fully qualified pathnames
	of every file that was unlocked. Directories will be skipped.

	Examples:
		# Any file that is read only in this directory is now unlocked
		lock_list = unlock_files("~/projects/lockedfiles")

		# Do stuff on the files
		do_code_on_unlocked_files()

		# Re-lock all the files that were unlocked.
		lock_files(lock_list)

	Args:
		working_dir: Pathname to the directory to traverse for read-only files
		recursive: False (default) don't recurse through folders, True, recurse
	Returns:
		A list object with the name of every file that was unlocked.

	See:
		lock_files()

	"""

	# Ensure the pathname is unmangled
	abs_dir = os.path.abspath(working_dir)

	# Iterate over the directory
	result = []
	for item in os.listdir(abs_dir):
		path_name = os.path.join(abs_dir, item)

		# Get the status of the file
		path_stat = os.stat(path_name)

		# Process files
		if path_stat.st_mode & stat.S_IFREG:

			# Only care about write protected files
			if not path_stat.st_mode & stat.S_IWRITE:

				# Remove write protection while retaining the other flags
				os.chmod(path_name, path_stat.st_mode + stat.S_IWRITE + \
					stat.S_IWGRP + stat.S_IWOTH)
				result.append(path_name)
		else:
			# Process recursion
			if recursive and path_stat.st_mode & stat.S_IFDIR:
				result += unlock_files(path_name, True)
	return result


########################################


def lock_files(lock_list):
	"""
	Iterate over the input list and mark all files as read-only

	Args:
		lock_list: Iterable object containing a list of path names to files
			or directories to mark as "read-only"
	See:
		unlock_files()
	"""
	for item in lock_list:

		# Get the status of the file
		path_stat = os.stat(item)

		# Mark it write protected for Perforce
		os.chmod(item, path_stat.st_mode & \
			(~(stat.S_IWRITE + stat.S_IWGRP + stat.S_IWOTH)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Package that contains string manipulation functions

"""

## \package burger.strutils

from __future__ import absolute_import, print_function, unicode_literals

import sys
import os
import string

########################################


def unicode_print(input_string):
	"""
	Handle printing a unicode string to stdout

	On some platforms, printing a unicode string will trigger
	a UnicodeEncodeError exception. In these cases, handle the
	exception and recode the string to the native string
	encoding.

	Args:
		input_string: A unicode string to print to stdout.
	"""

	# Print the string, if no exception, exit
	try:
		print(input_string)
	except UnicodeEncodeError:
		# Ensure it's encoded to utf-8
		encoded = input_string.encode('utf-8')
		if sys.version_info[0] == 2:

			# Python 2.x only accepts this as input
			print(encoded)
		else:

			# Python 3.x and higher will allow remapping to
			# selected character encoding
			print(encoded.decode(sys.stdout.encoding))

########################################


try:
	# Test if basestring exists (Only in Python 2.x)
	basestring										# pylint: disable=W0104,basestring-builtin
	_IS_STRING_TEST = basestring					# pylint: disable=C0103,basestring-builtin

except NameError:
	# Python 3 or later
	_IS_STRING_TEST = (str, bytes)


def is_string(item):
	"""
	Return True if input is a string object

	Test the input if it's either an instance of
	basestring in Python 2.x or (str, bytes) in Python 3.x

	Args:
		item: Object to test
	Returns:
		True if the object is a string instance, False if not.
	"""

	return isinstance(item, _IS_STRING_TEST)

########################################


def convert_to_array(input_array):
	"""
	Convert a string to a string array (list)

	If the input is None, return an empty list. If
	it's a string, convert the string to a single entry list.
	Otherwise, assume it's an iterable dir, list or tuple of
	strings.

	Args:
		input_array: The object to test
	Returns:
		The input, or a string encapsulated into a single entry list.

	"""

	# If empty, return an empty array
	if input_array is None:
		input_array = []
	elif is_string(input_array):
		# Convert a single entry into an array
		input_array = [input_array]
	return input_array

########################################


def TrueFalse(item):			# pylint: disable=C0103
	"""
	Convert the input into a boolean and return the string 'True' or 'False'

	If the input was a string of '0' or 'False' (Case insensitive comparision),
	this function will return 'False'. Empty dictionary, string or list objects,
	or the number zero will also return 'False'

	Args:
		item: Object to convert to a bool before converting into a string
	Returns:
		The string 'True' or 'False'
	See:
		truefalse() or TRUEFALSE()
	"""

	# Test if it's the string 'False'
	if is_string(item):
		if item == '0' or item.upper() == 'FALSE':
			return 'False'
	if bool(item):
		return 'True'
	return 'False'

########################################


def truefalse(item):
	"""
	Convert the input into a boolean and return the string 'true' or 'false'

	If the input was a string of '0' or 'False' (Case insensitive comparision),
	this function will return 'false'. Empty dictionary, string or list objects,
	or the number zero will also return 'false'

	Args:
		item: Object to convert to a bool before converting into a string
	Returns:
		The string 'true' or 'false'
	See:
		TRUEFALSE() or TrueFalse()
	"""

	# Test if it's the string 'False'
	if is_string(item):
		if item == '0' or item.upper() == 'FALSE':
			return 'false'
	if bool(item):
		return 'true'
	return 'false'

########################################


def TRUEFALSE(item):			# pylint: disable=C0103
	"""
	Convert the input into a boolean and return the string 'TRUE' or 'FALSE'

	If the input was a string of '0' or 'False' (Case insensitive comparision),
	this function will return 'FALSE'. Empty dictionary, string or list objects,
	or the number zero will also return 'FALSE'

	Args:
		item: Object to convert to a bool before converting into a string
	Returns:
		The string 'TRUE' or 'FALSE'
	See:
		truefalse() or TrueFalse()
	"""

	# Test if it's the string 'False'
	if is_string(item):
		if item == '0' or item.upper() == 'FALSE':
			return 'FALSE'
	if bool(item):
		return 'TRUE'
	return 'FALSE'

########################################


def convert_to_windows_slashes(path_name, force_ending_slash=False):
	"""
	Convert a filename from Linux/macOS to Windows format

	Convert all '/' characters into '\' characters

	If force_ending_slash is True, append a '\' if one is not
	present in the final string

	Args:
		path_name: A pathname to be converted to Windows slashes
		force_ending_slash: True if a '\\' character is to be forced at the end
			of the output

	Returns:
		A pathname using Windows type slashes '\\'

	See:
		convert_to_linux_slashes()

	"""

	result = path_name.replace('/', '\\')
	if force_ending_slash and not result.endswith('\\'):
		result = result + '\\'
	return result

########################################


def convert_to_linux_slashes(path_name, force_ending_slash=False):
	"""
	Convert a filename from Windows to Linux/macOS format

	Convert all '\' characters into '/' characters

	Args:
		path_name: A string object that text substitution will occur
		force_ending_slash: True if a '/' character is to be forced at the end
			of the output

	Returns:
		A pathname using Linux/BSD type slashes '/'

	See:
		convert_to_windows_slashes()
	"""

	result = path_name.replace('\\', '/')
	if force_ending_slash and not result.endswith('/'):
		result = result + '/'
	return result

########################################


_WINDOWSSAFESET = frozenset(string.ascii_letters + string.digits + '_-.:\\')


def encapsulate_path_windows(input_path):

	"""
	Quote a pathname for use in the Windows system shell

	On Windows platforms, if the path has a space or other
	character that could confuse COMMAND.COM, the string
	will be quoted and double quotes within the string handled
	properly. All slash characters will be replaced with backslash
	characters.

	Args:
		input_path: string with the path to encapsulate using Windows rules
	Returns:
		Original input string if Windows can accept it or input properly quoted
	See:
		encapsulate_path()
	"""

	# Force to Windows slashes
	temp = convert_to_windows_slashes(input_path)

	# If there are any illegal characters, break
	for item in temp:
		if item not in _WINDOWSSAFESET:
			break
	else:
		# No illegal characters in the string
		if not temp:
			return '""'
		return temp

	# Since the test failed, quote the string
	return '"{}"'.format(temp.replace('"', '\\"'))

########################################


_LINUXSAFESET = frozenset(string.ascii_letters + string.digits + '@%_-+=:,./')


def encapsulate_path_linux(input_path):
	"""
	Quote a pathname for use in the linux or BSD system shell

	On Linux platforms, if the path has a space or other
	character that could confuse bash, the string
	will be quoted and double quotes within the string handled
	properly. All backslash characters will be replaced with slash
	characters.

	Args:
		input_path: string with the path to encapsulate using Windows rules
	Returns:
		Original input string if Windows can accept it or input properly quoted
	See:
		encapsulate_path()
	"""

	# Force to linux slashes
	temp = convert_to_linux_slashes(input_path)

	# If there are illegal characters for linux/BSD, break
	for item in temp:
		if item not in _LINUXSAFESET:
			break
	else:
		# String doesn't need quotes
		if not temp:
			return "''"
		return temp
	# Enquote the string for Linux or MacOSX
	return "'{}'".format(temp.replace("'", "'\"'\"'"))

########################################


def encapsulate_path(input_path):

	"""
	Quote a pathname for use in the native system shell

	On Windows platforms, if the path has a space or other
	character that could confuse COMMAND.COM, the string
	will be quoted, and for other platforms, it will
	be quoted using rules that work best for BASH.
	This will also quote if the path has a ';' which
	could be used to confuse bash.

	Args:
		input_path: string with the path to encapsulate
	Returns:
		Input string or input properly quoted
	See:
		encapsulate_path_windows() and encapsulate_path_linux()
	"""

	# Process for Windows platforms
	if os.name == 'nt':
		return encapsulate_path_windows(input_path)

	# Force to linux slashes
	return encapsulate_path_linux(input_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Package that contains build helper functions
"""

## \package burger.buildutils

from __future__ import absolute_import, print_function, unicode_literals

import errno
import os
import platform
import subprocess
import sys
from .strutils import is_string

# pylint: disable=C0302

# Use the old way for Python 2 versus 3
_PY2 = sys.version_info[0] == 2
if _PY2:
	from cStringIO import StringIO
else:
	from io import StringIO

# Redefining built-in W0622 (Ignore redefinition of zip)

try:
	import itertools.izip as zip		# pylint: disable=W0622
except ImportError:
	pass

## Cached location of the BURGER_SDKS folder
_BURGER_SDKS_FOLDER = None

## Cached location of doxygen
_DOXYGEN_PATH = None

## Cached location of p4 from Perforce
_PERFORCE_PATH = None

## Environment variable locations of window applications
_WINDOWS_ENV_PATHS = [
	'ProgramFiles',
	'ProgramFiles(x86)'
]

########################################


def get_sdks_folder(verbose=False, refresh=False, folder=None):
	"""
	Return the path of the BURGER_SDKS folder

	If the environment variable BURGER_SDKS is set,
	return the pathname it contains. Otherwise,
	print a warning if verbose is True and then attempt to find
	the 'sdks' folder by traversing the current working directory
	for a folder named 'sdks'. If one isn't found, return None.

	Examples:
		# Normal use
		sdksfolder = burger.buildutils.get_sdks_folder()
		if not sdksfolder:
			print('failure')
			raise NameError("sdks not found, set BURGER_SDKS")

		# Alert the user if BURGER_SDKS isn't set
		burger.buildutils.get_sdks_folder(verbose=True)

		# Force the use of a supplied folder for sdks
		burger.buildutils.get_sdks_folder(refresh=True, folder='./foo/sdks/')

	Args:
		verbose: If True, print a message if BURGER_SDKS was not present
		refresh: If True, reset the cache and force a reload.
		folder: Path to use as BURGER_SDKS in the cache as an override

	Returns:
		None if the environment variable is not set, or the
		value of BURGER_SDKS.
	"""

	global _BURGER_SDKS_FOLDER				# pylint: disable=W0603

	# Clear the cache if needed
	if refresh:
		_BURGER_SDKS_FOLDER = None

	# Set the override, if found
	if folder:
		_BURGER_SDKS_FOLDER = folder

	# Not cached?
	if _BURGER_SDKS_FOLDER is None:

		# Load from the system
		_BURGER_SDKS_FOLDER = os.getenv('BURGER_SDKS', default=None)

		# Test for None or empty string
		if not _BURGER_SDKS_FOLDER:

			# Warn about missing environment variable
			if verbose:
				print('The environment variable "BURGER_SDKS" is not set')

			# Try to find the directory in the current path
			from .fileutils import traverse_directory
			sdks = traverse_directory(os.getcwd(), 'sdks', \
				find_directory=True, terminate=True)
			if sdks:
				_BURGER_SDKS_FOLDER = sdks[0]
				if verbose:
					print('Assuming {} is the BURGER_SDKS folder'.format(sdks[0]))

	return _BURGER_SDKS_FOLDER

########################################


def host_machine():
	"""
	Return the high level operating system's name

	Return the machine this script is running on, 'windows', 'macosx',
	'linux' or 'unknown'

	Returns:
		The string 'windows', 'macosx', 'linux', or 'unknown'

	See:
		get_mac_host_type() or get_windows_host_type()
	"""
	# Only windows reports as NT

	if os.name == 'nt':
		return 'windows'

	# BSD and GNU report as posix

	if os.name == 'posix':

		# MacOSX is the Darwin kernel

		if platform.system() == 'Darwin':
			return 'macosx'

		# Assume linux (Tested on Ubuntu and Red Hat)

		return 'linux'

	# Surrender Dorothy

	return 'unknown'

########################################


def fix_csharp(csharp_application_path):
	"""
	Convert pathname to execute a C# exe file

	C# applications can launch as is on Windows platforms,
	however, on Mac OSX and Linux, it must be launched
	from mono. Determine the host machine and if not
	windows, automatically prepend 'mono' to
	the application's name to properly launch it

	This will also encase the name in quotes in case there are
	spaces in the pathname

	Args:
		csharp_application_path: Pathname string to update

	Returns:
		Command line appropriate for the platform to launch a C# application.
	"""

	# Encapsulate in quotes if needed
	from .strutils import encapsulate_path
	quoted = encapsulate_path(csharp_application_path)

	# Prepend mono on non-windows systems
	if host_machine() != 'windows':
		return 'mono {}'.format(quoted)
	return quoted


########################################


def get_windows_host_type():
	"""
	Return windows host type (32 or 64 bit)

	Return False if the host is not Windows, 'x86' if it's a 32 bit host
	and 'x64' if it's a 64 bit host, and possibly 'arm' if an arm host

	Returns:
		The string 'x64', 'x86', 'arm' or False
	See:
		get_mac_host_type() or host_machine()

	"""

	# Not windows?

	if os.name != 'nt':
		return False

	# Test the CPU for the type

	machine = platform.machine()
	if machine in ('AMD64', 'x86_64'):
		return 'x64'
	return 'x86'

########################################


def get_mac_host_type():
	"""
	Return Mac OSX host type (PowerPC/Intel)

	Return False if the host is not Mac OSX. 'ppc' if it's a Power PC based
	system, 'x86' for Intel (Both 32 and 64 bit)

	Returns:
		The string 'x86', 'ppc' or False

	See:
		get_windows_host_type() or host_machine()
	"""

	# Mac/Linux?
	if os.name != 'posix':
		return False

	# Not linux?

	if platform.system() != 'Darwin':
		return False

	# Since it's a mac, query the Mac OSX cpu type
	# using the MacOSX python extensions

	cpu = platform.machine()
	if cpu in ('x86', 'x86_64'):
		return 'x86'

	if cpu in ('PowerPC', 'ppc', 'Power Macintosh'):
		return 'ppc'

	# Defaults to PowerPC
	return 'ppc'

########################################


def is_exe(exe_path):
	"""
	Return True if the file is executable

	Note:
		Windows platforms don't support the 'x' bit so all
		files are executable if they exist.

	Args:
		exe_path: Full or partial pathname to test for existance
	Returns:
		True if the file is executable, False if the file doesn't exist or
		is not valid.
	"""
	return os.path.isfile(exe_path) and os.access(exe_path, os.X_OK)

########################################


def get_path_ext(pathext=None):
	"""
	Return a list of executable extensions

	If pathext is None, query the environment variable PATHEXT and
	return the entries as a string list. If pathext is a string,
	parse it as if it was a system specific PATHEXT string and
	if it's an iterable, return the value as is. If PATHEXT doesn't exist
	or is empty, return an empty list.

	Windows usually sets the value like this
	``PATHEXT=.COM;.EXE;.BAT;.CMD;.VBS;.VBE;.JS;.JSE;.WSF;.WSH;.MSC``

	Args:
		pathext: String parsed as PATHEXT, iterable returned as is
	Returns:
		List of file name extension strings.
	See:
		burger.buildutils.make_exe_path() or burger.buildutils.find_in_path()
	"""

	# Read the environment variable?
	if pathext is None:
		pathext = os.getenv('PATHEXT', [])

	# If a string, or environment variable?
	if is_string(pathext):
		# Parse the string
		pathext = pathext.split(os.pathsep)

	# Return the list or iterable
	return pathext

########################################


def make_exe_path(exe_path, pathext=None):
	"""
	Given a folder and a executable name, return the true absolute path

	Examples:
		# exe could be returned as exe, exe.exe, exe.cmd, etc...
		path = make_exe_path('C:\\code\\exe')
		if path is None:
			print('No file named exe at C:\\code')

	Note:
		On macOS and Linux, PATHEXT is not set, this is for supporting
		extension types for common batch files or other executable extensions.

	Args:
		exe_path: Path of the executable to test
		pathext: Extension list to test
	Returns:
		None if a match was not found, or a full pathname with extension.

	See:
		burger.buildutils.get_path_ext() or burger.buildutils.find_in_path()
	"""
	# Test the path as is
	if is_exe(exe_path):
		return exe_path

	# Try all the extensions (Can be an empty list)
	for ext in get_path_ext(pathext):
		temp_path = exe_path + ext
		if is_exe(temp_path):
			break
	else:
		return None
	return temp_path

########################################


def find_in_path(filename, search_path=None, executable=False):
	"""
	Using the system PATH environment variable, search for a file

	If the flag executable is False, the file will be found using a
	simple path search. If the flag is True, the file will be searched
	for using the extensions in the PATHEXT environment variable in
	addition to use the filename as is.

	If search_path is a string, it will be seperated using os.pathsep. If
	not, it will be treated as an interable list of strings of full pathnames
	to search. If it is None, the PATH environment variable will be used.

	Examples:
		# Can return 'doxygen', 'doxygen.exe' or 'doxygen.com' depending
		# on what was found
		burger.find_in_path('doxygen', executable=True)

		# Will only find 'foo.txt'
		burger.find_in_path('foo.txt')

	Args:
		filename: File to locate
		search_path: Search paths to use instead of PATH
		executable: True to ensure it's an executable
	Return:
		None if not found, a full path if the file is found
	See:
		burger.buildutils.get_path_ext() or burger.buildutils.make_exe_path()
	"""

	# Set up for added standard extentions
	if executable:
		pathext = get_path_ext()
	else:
		pathext = []

	# Is there a search path override?
	if search_path is None:
		# Use the environment variable
		paths = os.getenv('PATH', [])
	else:
		paths = search_path

	if is_string(paths):
		# Break it up based on the path seperator
		paths = paths.split(os.pathsep)

	# Since PATH was used, also test the current working directory
	# first
	if search_path is None:
		paths.insert(0, os.getcwd())

	# Scan the list of paths to find the file
	for item in paths:
		# Get the full path
		temp_path = os.path.join(item, filename)

		# Perform the test as an exe
		if executable:
			temp_path = make_exe_path(temp_path, pathext=pathext)
			if temp_path:
				break
		# Test for just a file
		elif os.path.isfile(temp_path):
			break
	else:
		# Not found in the loops
		return None

	# Return the path
	return os.path.abspath(temp_path)

########################################


def expand_and_verify(file_string):
	"""
	Expand the input string with os.path.expandvars()

	After expanding the string, test for the existence of the file
	and return the expanded path if True. Otherwise, return None

	Examples:
		perforcepath = burger.expand_and_verify('${PERFORCE}\\p4.exe')
		if perforcepath is None:
			return

	Args:
		file_string: Pathname with environment variable tokens

	Returns:
		None if the string couldn't be expanded or if the file didn't exist,
			otherwise, return the expanded pathname

	"""

	result_path = os.path.expandvars(file_string)
	if result_path is not None:
		if not os.path.isfile(result_path):
			result_path = None
	return result_path

########################################


def where_is_doxygen(verbose=False, refresh=False, path=None):
	"""
	Return the location of Doxygen's executable

	Look for an environment variable DOXYGEN and
	determine if the executable resides there, if
	so, return the string to the path

	If running on a MacOSX client, look in the Applications
	folder for a copy of Doxygen.app and return the
	pathname to the copy of doxygen that resides within

	PATH is then searched for doxygen, and if it's not found,
	None is returned.

	Args:
		verbose: If True, print a message if doxygen was not found
		refresh: If True, reset the cache and force a reload.
		path: Path to doxygen to place in the cache

	Returns:
		A path to the Doxygen command line executable or None if not found.

	"""

	global _DOXYGEN_PATH				# pylint: disable=W0603

	# Clear the cache if needed
	if refresh:
		_DOXYGEN_PATH = None

	# Set the override, if found
	if path:
		_DOXYGEN_PATH = path

	# Is cached?
	if _DOXYGEN_PATH:
		return _DOXYGEN_PATH

	# Try the environment variable first
	if os.getenv('DOXYGEN', None):
		if get_windows_host_type():

			# Windows points to the base path
			doxygenpath = os.path.expandvars('${DOXYGEN}\\bin\\doxygen.exe')
		else:
			# Just append the exec name
			doxygenpath = os.path.expandvars('${DOXYGEN}/doxygen')

		# Valid?
		if is_exe(doxygenpath):
			_DOXYGEN_PATH = doxygenpath
			return doxygenpath

	# Scan the PATH for the exec
	doxygenpath = find_in_path('doxygen', executable=True)
	if doxygenpath:
		_DOXYGEN_PATH = doxygenpath
		return doxygenpath

	# List of the usual suspects
	full_paths = []

	# Check if it's installed but not in the path
	if get_windows_host_type():

		# Try the 'ProgramFiles' folders
		for item in _WINDOWS_ENV_PATHS:
			if os.getenv(item, None):
				full_paths.append(os.path.expandvars( \
					'${' + item + '}\\doxygen\\bin\\doxygen.exe'))

	elif get_mac_host_type():

		# MacOSX has it hidden in the application
		full_paths.append('/Applications/Doxygen.app/Contents/Resources/doxygen')
		full_paths.append('/opt/local/bin/doxygen')

	elif os.name == 'posix':
		# Posix / Linux
		full_paths.append('/usr/bin/doxygen')

	# Scan the list of known locations
	for doxygenpath in full_paths:
		if is_exe(doxygenpath):
			# Finally found it!
			_DOXYGEN_PATH = doxygenpath
			return doxygenpath

	# Oh, dear.
	if verbose:
		print('Doxygen not found!')
		if get_mac_host_type():
			print('Install the desktop application in the Applications folder or ' \
				'use brew or macports for the command line version')

	# Can't find it
	return None

########################################


def where_is_p4(verbose=False, refresh=False, path=None):

	"""
	Return the location of the p4 executable

	Look for an environment variable PERFORCE and
	determine if the executable resides there, if
	so, return the string to the path.

	PATH is then searched for p4, and if it's not found,
	None is returned.

	Args:
		verbose: If True, print a message if Perforce was not found
		refresh: If True, reset the cache and force a reload.
		path: Path to Perforce to place in the cache
	Returns:
		A path to the Perforce command line executable or None if not found.
	See:
		perforce_edit()
	"""

	global _PERFORCE_PATH				# pylint: disable=W0603

	# Clear the cache if needed
	if refresh:
		_PERFORCE_PATH = None

	# Set the override, if found
	if path:
		_PERFORCE_PATH = path

	# Is cached?
	if _PERFORCE_PATH:
		return _PERFORCE_PATH

	# Try the environment variable first
	if os.getenv('PERFORCE', None):
		if get_windows_host_type():

			# Windows points to the base path
			p4path = os.path.expandvars('${PERFORCE}\\p4.exe')
		else:
			# Just append the exec name
			p4path = os.path.expandvars('${PERFORCE}/p4')

		# Valid?
		if is_exe(p4path):
			_PERFORCE_PATH = p4path
			return p4path

	# Scan the PATH for the exec
	p4path = find_in_path('p4', executable=True)
	if p4path:
		_PERFORCE_PATH = p4path
		return p4path

	# List of the usual suspects
	full_paths = []

	# Check if it's installed but not in the path
	if get_windows_host_type():

		# Try the 'ProgramFiles' folders
		for item in _WINDOWS_ENV_PATHS:
			if os.getenv(item, None):
				full_paths.append(os.path.expandvars( \
					'${' + item + '}\\perforce\\p4.exe'))

	elif get_mac_host_type():

		# Installed here via brew
		full_paths.append('/opt/local/bin/p4')

	elif os.name == 'posix':
		# Posix / Linux
		full_paths.append('/usr/bin/p4')

	# Scan the list of known locations
	for p4path in full_paths:
		if is_exe(p4path):
			# Finally found it!
			_PERFORCE_PATH = p4path
			return p4path

	# Oh, dear.
	if verbose:
		print('Perforce "p4" not found!')
		if get_mac_host_type():
			print('Use brew or macports for the command line version')

	# Can't find it
	return None

########################################


def perforce_edit(files, verbose=False):

	"""
	Given a list of files, checkout (Edit) them in perforce

	Pass either a single string or a string list of pathnames
	of files to checkout in perforce using the 'p4 edit' command

	Args:
		files: list or string object of file(s) to checkout
		verbose: If True, print the command line and warnings

	Returns:
		Zero if no error, non-zero on error
	See:
		where_is_p4()
	"""

	# Get the p4 executable
	perforce_path = where_is_p4(verbose=verbose)

	# Not found?
	if perforce_path is None:
		return 10

	# Encapsulate the single string entry
	if is_string(files):
		file_list = (files,)
	else:
		file_list = files

	# Generate the command line and call
	from .strutils import encapsulate_path
	p4quoted = encapsulate_path(perforce_path)
	error = 0
	for item in file_list:
		cmd = '{} edit {}'.format(p4quoted, encapsulate_path(os.path.abspath(item)))
		if verbose:
			print(cmd)
		error = subprocess.call(cmd, shell=True)
		if error != 0:
			break
	return error

########################################


def compare_files(filename1, filename2):
	"""
	Compare text files for equality

	Check if two text files are the same length,
	and then test the contents to verify equality.

	Args:
		filename1: string object with the pathname of the file to test
		filename2: string object with the pathname of the file to test against

	Returns:
		True if the files are equal, False if not.

	See:
		compare_file_to_string()
	"""

	# Load in the two text files

	try:
		with open(filename1, 'r') as filep:
			file_one_lines = filep.read().splitlines()
		with open(filename2, 'r') as filep:
			file_two_lines = filep.read().splitlines()
	except IOError as error:
		# Only deal with file not found
		if error.errno != errno.ENOENT:
			raise
		# If not found, return "not equal"
		return False

	del filep

	# Compare the file contents

	if len(file_one_lines) == len(file_two_lines):
		for i, j in zip(file_one_lines, file_two_lines):
			if i != j:
				break
		else:
			# It's a match!
			return True
	return False

########################################


def compare_file_to_string(filename, data):

	"""
	Compare text file and a string for equality

	Check if a text file is the same as a string by loading the text file and
	testing line by line to verify the equality of the contents

	Args:
		filename: string object with the pathname of the file to test
		data: string object to test against

	Returns:
		True if the file and the string are the same, False if not

	See:
		compare_files()
	"""

	# Do a data compare as a text file

	try:
		with open(filename, 'r') as filep:
			file_one_lines = filep.read().splitlines()
	except IOError as error:
		# Only deal with file not found
		if error.errno != errno.ENOENT:
			raise
		# If not found, return "not equal"
		return False

	del filep

	# Compare the file contents taking into account
	# different line endings

	# Test if this is a StringIO object
	if hasattr(data, 'getvalue'):
		file_two_lines = data.getvalue().splitlines()
	else:
		file_two_lines = data.splitlines()

	# Compare the file contents

	if len(file_one_lines) == len(file_two_lines):
		for i, j in zip(file_one_lines, file_two_lines):
			if i != j:
				break
		else:
			# It's a match!
			return True
	return False

########################################


def run_command(args, working_dir=None, quiet=False):
	"""
	Execute a program and capture the return code and text output

	Pass a command line formatted for the current shell and then this
	function will execute that command and capture both stdout and stderr.

	Args:
		args: List of command line entries, starting with the program pathname
		working_dir: Directory to set before executing command
		quiet: Set to True if errors should not be printed

	Returns:
		The return code, stdout, stderr
	"""
	try:
		tempfp = subprocess.Popen(args, cwd=working_dir, stdout=subprocess.PIPE, \
			stderr=subprocess.PIPE, universal_newlines=True)
	except OSError as error:
		if not quiet:
			print('Command line "{}" generated error {}'.format(args, error))
		return (error.errno, '', '')

	stdoutstr, stderrstr = tempfp.communicate()
	return (tempfp.returncode, stdoutstr, stderrstr)

########################################


def make_version_header(working_dir, outputfilename, verbose=False):
	"""
	Create a C header with the perforce version

	This function assumes version control is with perforce!

	Get the last change list and create a header
	with this information (Only modify the output file if
	the contents have changed)

	C++ defines are declared for P4_CHANGELIST, P4_CHANGEDATE, P4_CHANGETIME,
	P4_CLIENT, and P4_USER

	Args:
		working_dir: string with the path of the folder to obtain the perforce
			version for
		outputfilename: string with the path of the generated header
		verbose: Print perforce commands and other informational messages

	Returns:
		Zero if no error, non-zero on error
	"""

	# Check if perforce is installed
	p4exe = where_is_p4()
	if p4exe is None:
		return 10
	# Encapsulate in quotes if needed
	from .strutils import encapsulate_path
	p4cmd = encapsulate_path(p4exe)

	# Create the header guard by taking the filename,
	# converting to upper case and replacing spaces and
	# periods with underbars.
	headerguard = os.path.basename(outputfilename).upper()
	headerguard = headerguard.replace(' ', '_')
	headerguard = '__{}__'.format(headerguard.replace('.', '_'))

	# Get the last change list
	# Parse "Change 3361 on 2012/05/15 13:20:12 by burgerbecky@burgeroctocore
	# 'Made a p4 change'"
	# -m 1 / Limit to one entry
	# -t / Display the time
	# -l / Print out the entire changelist description

	cmd = "{} changes -m 1 -t -l ...#have".format(p4cmd)
	if verbose:
		print(cmd)
	error, tempdata = run_command(cmd, working_dir)[:2]
	if error != 0:
		return error

	# Parse out the output of the p4 changes command
	p4changes = tempdata.strip().split(' ')

	# Get the p4 client
	# Parse "P4CLIENT=burgeroctocore (config)"

	cmd = "{} set P4CLIENT".format(p4cmd)
	if verbose:
		print(cmd)
	error, tempdata = run_command(cmd, working_dir)[:2]
	if error != 0:
		return error

	# Parse out the P4CLIENT query
	p4clients = tempdata.strip().split(' ')[0].split('=')

	# Get the p4 user
	# Parse "P4USER=burgerbecky (config)"

	cmd = "{} set P4USER".format(p4cmd)
	if verbose:
		print(cmd)
	error, tempdata = run_command(cmd, working_dir)[:2]
	if error != 0:
		return error

	# Parse out the P4USER query
	p4users = tempdata.strip().split(' ')[0].split('=')

	# Write out the header

	filep = StringIO()
	filep.write( \
		'/***************************************\n'
		'\n'
		'\tThis file was generated by a call to\n'
		'\tburger.make_version_header() from a build\n'
		'\tpython script\n'
		'\n'
		'***************************************/\n'
		'\n'
		'#ifndef {0}\n'
		'#define {0}\n'
		'\n'.format(headerguard))

	if len(p4changes) > 4:
		filep.write('#define P4_CHANGELIST ' + p4changes[1] + '\n')
		filep.write('#define P4_CHANGEDATE "' + p4changes[3] + '"\n')
		filep.write('#define P4_CHANGETIME "' + p4changes[4] + '"\n')

	if len(p4clients) > 1:
		filep.write('#define P4_CLIENT "' + p4clients[1] + '"\n')

	if len(p4users) > 1:
		filep.write('#define P4_USER "' + p4users[1] + '"\n')

	filep.write('\n#endif\n')

	# Check if the data is different than what's already stored on
	# the drive

	filevalue = filep.getvalue()
	del filep

	if compare_file_to_string(outputfilename, filevalue) is not True:
		if verbose:
			print('Writing ' + outputfilename)
		try:
			with open(outputfilename, 'w') as filep:
				filep.write(filevalue)
		except IOError as error:
			print(error)
			return 2
	return 0

########################################


def is_codewarrior_mac_allowed():
	"""
	Return True if this machine can run Codewarrior for Mac OS Carbon

	Test first if the host platform is a mac, and if so, test if it's
	capable of running Mac OS Carbon Codewarrior 9 or 10

	Returns:
		True if CodeWarrior for Mac OS can be run on this Macintosh

	See:
		host_machine()
	"""

	# Test if a mac

	if get_mac_host_type():
		# Get the Mac OS version number
		mac_ver = platform.mac_ver()
		release = mac_ver[0]

		# Convert 10.5.8 to 10.5

		digits = release.split('.')

		# Snow Leopard (10.6) supports Rosetta
		# Lion (10.7) and Mountain Lion (10.8) do not

		if float(digits[0]) == 10:
			if float(digits[1]) < 7:
				return True

	# Can't run, not a mac or Power PC native or emulation isn't supported
	return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A set of subroutines used by the Burgerlib based scripts written in Python.

"""

# pylint: disable=C0302

#
## \package burger
#
# For higher level tools like makeprojects, cleanme and
# buildme, common subroutines were collected and
# placed in this module for reuse.
#

#
## \mainpage
#
# \htmlinclude README.html
#
#
# Module list
# ===========
#
# - \ref burger
# - \ref burger.buildutils
# - \ref burger.fileutils
# - \ref burger.strutils
#

from __future__ import absolute_import, print_function, unicode_literals

import sys

from .__pkginfo__ import NUMVERSION, VERSION, AUTHOR, TITLE, SUMMARY, URI, \
	EMAIL, LICENSE, COPYRIGHT
from .strutils import unicode_print, is_string, convert_to_array, TrueFalse, \
	truefalse, TRUEFALSE, convert_to_windows_slashes, convert_to_linux_slashes, \
	encapsulate_path_windows, encapsulate_path_linux, encapsulate_path
from .fileutils import create_folder_if_needed, delete_file, is_source_newer, \
	copy_file_if_needed, copy_file_checkout_if_needed, \
	copy_directory_if_needed, shutil_readonly_cb, delete_directory, \
	get_tool_path, traverse_directory, unlock_files, lock_files
from .buildutils import StringIO, get_sdks_folder, host_machine, fix_csharp, \
	get_windows_host_type, get_mac_host_type, is_exe, get_path_ext, \
	make_exe_path, find_in_path, where_is_doxygen, where_is_p4, \
	expand_and_verify, perforce_edit, compare_files, compare_file_to_string, \
	run_command, make_version_header, is_codewarrior_mac_allowed

########################################


## Numeric version
__numversion__ = NUMVERSION

## Current version of the library
__version__ = VERSION

## Author's name
__author__ = AUTHOR

## Name of the module
__title__ = TITLE

## Summary of the module's use
__summary__ = SUMMARY

## Home page
__uri__ = URI

## Email address for bug reports
__email__ = EMAIL

## Type of license used for distribution
__license__ = LICENSE

## Copyright owner
__copyright__ = COPYRIGHT

#
## Items to import on "from burger import *"
#

__all__ = [
	'unicode_print',
	'is_string',
	'convert_to_array',
	'TrueFalse',
	'truefalse',
	'TRUEFALSE',
	'convert_to_windows_slashes',
	'convert_to_linux_slashes',
	'encapsulate_path_windows',
	'encapsulate_path_linux',
	'encapsulate_path',
	'create_folder_if_needed',
	'delete_file',
	'is_source_newer',
	'copy_file_if_needed',
	'copy_file_checkout_if_needed',
	'copy_directory_if_needed',
	'shutil_readonly_cb',
	'delete_directory',
	'get_tool_path',
	'traverse_directory',
	'unlock_files',
	'lock_files',
	'get_sdks_folder',
	'host_machine',
	'fix_csharp',
	'get_windows_host_type',
	'get_mac_host_type',
	'is_exe',
	'get_path_ext',
	'make_exe_path',
	'find_in_path',
	'where_is_doxygen',
	'expand_and_verify',
	'where_is_p4',
	'perforce_edit',
	'compare_files',
	'compare_file_to_string',
	'run_command',
	'make_version_header',
	'is_codewarrior_mac_allowed',
	'Interceptstdout',
	'Node'
]

########################################


class Interceptstdout(list):
	"""
	Handy class for capturing stdout from tools and	python itself

	Examples:
		# Import the class
		from burger import Interceptstdout

		# Instanciate the class, which intercepts stdout
		with Interceptstdout() as output:
			do_somethingthatprints()
			print('capture me!')

		# Once out of scope, output has a list of strings
		# of the captured stdout output.
		print(output)

	"""

	def __init__(self):
		"""
		Declares the internal variables
		"""

		## Saved copy of sys.stdout
		self._stdout = None

		## StringIO to redirect output to
		self._stringio = None
		super(Interceptstdout, self).__init__()

	def __enter__(self):
		"""
		Invoked on 'with' which intercepts all future stdout
		"""

		self._stdout = sys.stdout
		sys.stdout = self._stringio = StringIO()
		return self

	def __exit__(self, *args):
		"""
		Disconnect the stdout and store the items into a list of lines

		Using splitlines(), output the buffer into a list of lines
		into the output field
		"""

		# Restore stdout on exit
		self.extend(self._stringio.getvalue().splitlines())
		sys.stdout = self._stdout
		self._stringio = None
		self._stdout = None

########################################


class Node(object):
	"""
	Node class for creating directory trees

	Needed for some projects that have to store
	file entries in nested trees
	"""

	# Too few public methods R0903
	# #pylint: disable=R0903

	def __init__(self, value, children=None):
		"""
		Create a node with an initial value

		Args:
			value: Object to be the value of this node
			children: Array of nodes to be added as children to this one
		"""

		if children is None:
			children = []

		## Value contained in this node
		self.value = value
		## Array of children nodes to this node
		self.children = children

	def __repr__(self, level=0):
		"""
		Display this node as a string

		Args:
			level: Recursion depth (Used internally)
		"""

		ret = '\t' * level + repr(self.value) + '\n'
		for child in self.children:
			ret += child.__repr__(level + 1)
		return ret

	__str__ = __repr__



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Build the egg file for burger for python

setup.py clean
setup.py build
setup.py sdist
setup.py upload

Copyright 2013-2018 by Rebecca Ann Heineman becky@burgerbecky.com

It is released under an MIT Open Source license. Please see LICENSE
for license details. Yes, you can use it in a
commercial title without paying anything, just give me a credit.
Please? It's not like I'm asking you for money!

"""

from __future__ import absolute_import, print_function, unicode_literals
import io
import os
import sys
import setuptools

CWD = os.path.dirname(os.path.abspath(__file__))

# Project specific strings
PROJECT_NAME = 'burger'
PROJECT_KEYWORDS = [
	'burger',
	'perforce',
	'burgerlib',
	'development'
]

# Manually import the project
PROJECT_MODULE = __import__(PROJECT_NAME)

# Read me file is the long description
with io.open(os.path.join(CWD, 'README.rst'), encoding='utf-8') as filep:
	LONG_DESCRIPTION = filep.read()

# Create the dependency list
INSTALL_REQUIRES = [
	'setuptools >= 0.7.0'
]

# Project classifiers
CLASSIFIERS = [
	'Development Status :: 5 - Production/Stable',
	'Environment :: Console',
	'Intended Audience :: Developers',
	'Topic :: Software Development',
	'Topic :: Software Development :: Build Tools',
	'License :: OSI Approved :: MIT License',
	'Operating System :: OS Independent',
	'Natural Language :: English',
	'Programming Language :: Python',
	'Programming Language :: Python :: 2',
	'Programming Language :: Python :: 2.7',
	'Programming Language :: Python :: 3',
	'Programming Language :: Python :: 3.3',
	'Programming Language :: Python :: 3.4',
	'Programming Language :: Python :: 3.5',
	'Programming Language :: Python :: 3.6'
]

# Extra files to include in the form of this tuple (directory,[files])
DATA_FILES = [
	# (PROJECT_NAME, ['LICENSE.txt'])
]

#
# Parms for setup
#

SETUP_ARGS = dict(
	name=PROJECT_NAME,
	version=PROJECT_MODULE.__version__,

	# Use the readme as the long description
	description=PROJECT_MODULE.__summary__,
	long_description=LONG_DESCRIPTION,
	# long_description_content_type='text/x-rst; charset=UTF-8',
	license=PROJECT_MODULE.__license__,
	url=PROJECT_MODULE.__uri__,

	author=PROJECT_MODULE.__author__,
	author_email=PROJECT_MODULE.__email__,

	keywords=PROJECT_KEYWORDS,
	platforms=['Any'],
	install_requires=INSTALL_REQUIRES,
	zip_safe=False,
	python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*',

	classifiers=CLASSIFIERS,
	packages=[PROJECT_NAME],
	include_package_data=True,
	data_files=DATA_FILES
)

########################################

CLEAN_DIR_LIST = [
	PROJECT_NAME + '.egg-info',
	PROJECT_NAME + '-' + PROJECT_MODULE.__version__,
	'dist',
	'build',
	'temp',
	'docs' + os.sep + 'temp',
	'_build',
	'__pycache__',
	'.pytest_cache',
	'.tox'
]

CLEAN_EXTENSION_LIST = [
	'.pyc',
	'.pyo'
]


def clean(working_dir):
	"""
	Clean up all the temp files after uploading

	Helps in keeping source control from having to track
	temp files
	"""

	# Delete all folders, including read only files

	for item in CLEAN_DIR_LIST:
		PROJECT_MODULE.delete_directory(os.path.join(working_dir, item))

	#
	# Delete all *.pyc and *.pyo files
	#

	name_list = os.listdir(working_dir)
	for base_name in name_list:
		file_name = os.path.join(working_dir, base_name)
		# Is it a file? (Skip directories)
		if os.path.isfile(file_name):
			for item in CLEAN_EXTENSION_LIST:
				if base_name.endswith(item):
					os.remove(file_name)
					break

#
# Perform the setup
#


if __name__ == '__main__':
	# Ensure the directory is the current one
	if CWD:
		os.chdir(CWD)

	# Perform a thorough cleaning job
	if 'clean' in sys.argv:
		clean(CWD)

	# Unlock the files to handle Perforce locking
	LOCK_LIST = PROJECT_MODULE.unlock_files(CWD) + \
		PROJECT_MODULE.unlock_files(os.path.join(CWD, PROJECT_NAME))
	try:
		setuptools.setup(**SETUP_ARGS)

	# If any files were unlocked, relock them
	finally:
		PROJECT_MODULE.lock_files(LOCK_LIST)
